from pathlib import Path
from typing import Generator
from urllib.parse import urlparse
import hashlib
import frontmatter
from chercher import Document, hookimpl


def normalize_uri(uri: str) -> Path:
    if uri.startswith("file://"):
        parsed_uri = urlparse(uri)
        return Path(parsed_uri.path).resolve()

    return Path(uri)


@hookimpl
def ingest(uri: str) -> Generator[Document, None, None]:
    path = normalize_uri(uri)
    if not path.exists() or not path.is_file() or path.suffix != ".md":
        return

    with path.open("rb") as f:
        content = f.read()
        hash = hashlib.sha256(content)
        post = frontmatter.loads(content.decode("utf-8"))

    yield Document(
        uri=path.as_uri(),
        title=path.name,
        body=post.content,
        hash=hash.hexdigest(),
        metadata={},
    )


@hookimpl()
def prune(uri: str) -> bool | None:
    if uri.startswith("file://"):
        parsed_uri = urlparse(uri)
        path = Path(parsed_uri.path).resolve()
    else:
        return

    if path.suffix != ".md":
        return

    if not path.exists() or not path.is_file():
        return True



from chercher_plugin_md import ingest, prune
from chercher import Document

CONTENT = """
---
title: TDD
---

# TDD
And how to do it in production.
"""


def test_ingest_valid_file_with_file_uri(tmp_path):
    p = tmp_path / "test.md"
    p.write_text(CONTENT)

    uri = p.as_uri()
    documents = list(ingest(uri=uri))
    assert len(documents) == 1
    for doc in documents:
        assert isinstance(doc, Document)
        assert doc.uri == uri
        assert doc.title == p.name
        assert doc.body != ""
        assert isinstance(doc.metadata, dict)
        assert doc.hash is not None


def test_ingest_valid_file_with_relative_uri(tmp_path):
    p = tmp_path / "test.md"
    p.write_text(CONTENT)

    uri = p.as_posix()
    documents = list(ingest(uri=uri))
    assert len(documents) == 1
    for doc in documents:
        assert isinstance(doc, Document)
        assert doc.uri == p.as_uri()
        assert doc.title == p.name
        assert doc.body != ""
        assert isinstance(doc.metadata, dict)
        assert doc.hash is not None


def test_ingest_invalid_file(tmp_path):
    p = tmp_path / "test.txt"
    p.write_text(CONTENT)

    uri = p.as_uri()
    documents = list(ingest(uri=uri))
    assert documents == []


def test_ingest_missing_file(tmp_path):
    p = tmp_path / "missingno.md"
    documents = list(ingest(uri=p.as_uri()))
    assert documents == []


def test_ingest_invalid_uri():
    uri = "https://blog/post.md"
    documents = list(ingest(uri=uri))
    assert documents == []


def test_prune_valid_file(tmp_path):
    p = tmp_path / "test.md"
    p.write_text(CONTENT)

    uri = p.as_uri()
    assert prune(uri=uri) is None


def test_prune_missing_file(tmp_path):
    p = tmp_path / "missingno.md"
    uri = p.as_uri()
    assert prune(uri=uri)


def test_prune_invalid_uri():
    uri = "https://files/file.md"
    assert prune(uri=uri) is None
